// src/utils/filesystem.ts
import fs from "fs";
import path from "path";
import { fileURLToPath } from "url";
function hasErrorCode(error, code) {
  return (error == null ? void 0 : error.code) === code;
}
async function fileExists(filePath) {
  try {
    await fs.promises.stat(filePath);
    return true;
  } catch (error) {
    if (hasErrorCode(error, "ENOENT")) {
      return false;
    }
    throw error;
  }
}
async function forEachDirentAsync(dirPath, action) {
  let dirFiles = [];
  try {
    dirFiles = await fs.promises.readdir(dirPath, { withFileTypes: true });
  } catch (error) {
  }
  await Promise.all(dirFiles.map(action));
}
async function readDirectory(dirPath) {
  try {
    const files = await fs.promises.readdir(dirPath);
    return files;
  } catch (error) {
    if (hasErrorCode(error, "ENOENT") || hasErrorCode(error, "ENOTDIR")) {
      return [];
    }
    throw error;
  }
}
async function removeEmptyDirsUp(checkedDirs, dirPath, count = 0) {
  if (!checkedDirs.has(dirPath)) {
    const files = await readDirectory(dirPath);
    const emptyDir = files.length === 0;
    checkedDirs.add(dirPath);
    if (emptyDir) {
      try {
        await fs.promises.rmdir(dirPath);
        count++;
      } catch (error) {
      }
      const parentDir = path.dirname(dirPath);
      count = await removeEmptyDirsUp(checkedDirs, parentDir, count);
    }
  }
  return count;
}
async function removeEmptyDirs(filePaths) {
  let removedEmptyDirs = 0;
  await Promise.all(
    filePaths.map(async (filePath) => {
      const removedParentDirs = await removeEmptyDirsUp(/* @__PURE__ */ new Set(), path.dirname(filePath));
      removedEmptyDirs += removedParentDirs;
    })
  );
  return removedEmptyDirs;
}
async function crawlDirFast(filePaths, dirPath) {
  await forEachDirentAsync(dirPath, async (dirent) => {
    const nextPath = dirPath + path.sep + dirent.name;
    if (dirent.isDirectory()) {
      await crawlDirFast(filePaths, nextPath);
    } else {
      filePaths.push(nextPath);
    }
  });
}
async function crawlDirWithChecks(filePaths, dirPath, checkDir, checkFile) {
  await forEachDirentAsync(dirPath, async (nextPathDirent) => {
    const nextPath = dirPath + path.sep + nextPathDirent.name;
    if (nextPathDirent.isDirectory()) {
      if (checkDir(nextPath)) {
        await crawlDirFast(filePaths, nextPath);
      } else {
        await crawlDirWithChecks(filePaths, nextPath, checkDir, checkFile);
      }
    } else if (checkFile(nextPath)) {
      filePaths.push(nextPath);
    }
  });
  return filePaths;
}
async function removeFiles(filePaths, options = {}) {
  let reducedSize = 0;
  await Promise.all(
    filePaths.map(async (filePath) => {
      try {
        const fileStats = await fs.promises.stat(filePath);
        if (!options.dryRun) {
          await fs.promises.unlink(filePath);
        }
        reducedSize += fileStats.size;
      } catch (error) {
      }
    })
  );
  return reducedSize;
}
function fileDir(importMetaUrl) {
  return path.dirname(fileURLToPath(importMetaUrl));
}

// src/shared.ts
import path2 from "path";
var DEFAULT_GLOBS_FILE_PATH = path2.resolve(
  fileDir(import.meta.url),
  "..",
  ".cleanmodules-default"
);
var sharedDefaultOptions = {
  directory: path2.resolve(process.cwd(), "node_modules"),
  globFile: ".cleanmodules",
  noDefaults: false,
  globs: []
};

// src/utils/glob.ts
import { promises as fsAsync } from "fs";
import path3 from "path";
import pm from "picomatch";
function initGlobLists() {
  return { excluded: [], included: [], includedDirs: [], originalIncluded: [] };
}
var DEFAULT_PICO_OPTIONS = {
  dot: true,
  nocase: true,
  strictSlashes: true
};
function makeGlobMatcher(globs, picoOptions) {
  return pm(globs, { ...DEFAULT_PICO_OPTIONS, ...picoOptions });
}
function updateGlobLists(globLists, updateFunc) {
  const { included, includedDirs, excluded } = globLists;
  return {
    ...globLists,
    included: updateFunc(included, "included"),
    includedDirs: includedDirs ? updateFunc(includedDirs, "includedDirs") : includedDirs,
    excluded: updateFunc(excluded, "excluded")
  };
}
function mergeGlobLists(globListsA, globListsB) {
  return {
    ...updateGlobLists(globListsA, (globs, key) => [...globs, ...globListsB[key] || []]),
    originalIncluded: [...globListsA.originalIncluded, ...globListsB.originalIncluded]
  };
}
function toPosixPath(pathStr) {
  return path3.sep === "/" ? pathStr : pathStr.replace(/\\/g, "/");
}
function toAbsoluteGlobLists(globLists, absoluteNodeModulesPath) {
  const absolutePathWithPosixSeparator = toPosixPath(absoluteNodeModulesPath);
  return updateGlobLists(
    globLists,
    (globs) => globs.map((glob) => absolutePathWithPosixSeparator + "/" + glob)
  );
}
function wrapGlobs(globs, prefix) {
  return `${prefix || ""}@(${globs.map((glob) => "(" + glob + ")").join("|")})`;
}
var GLOBSTAR_START_REGEX = /^(\/?\*\*\/)+/;
function optimizeGlobs(globs) {
  const globstarStartGlobs = [];
  const fixedPathGlobs = [];
  for (const glob of globs) {
    if (glob.match(GLOBSTAR_START_REGEX)) {
      globstarStartGlobs.push(glob);
    } else {
      fixedPathGlobs.push(glob);
    }
  }
  const result = [];
  if (globstarStartGlobs.length) {
    const withoutGlobstar = globstarStartGlobs.map((glob) => glob.replace(GLOBSTAR_START_REGEX, ""));
    result.push(wrapGlobs(withoutGlobstar, "**/"));
  }
  if (fixedPathGlobs.length) {
    result.push(wrapGlobs(fixedPathGlobs));
  }
  return result;
}
function optimizeGlobLists(globLists) {
  return updateGlobLists(globLists, optimizeGlobs);
}
var EXCLAMATION_START = /^\s*!/;
var DIR_GLOB_REGEX = /\/\**$/;
var EXCLUDED_GLOB_REGEX = /^!/;
var ESCAPED_NEGATIVE_GLOB_REGEX = /^\\!/;
var NOT_FIXED_START_GLOB_REGEX = /^([^/])/;
var FIXED_START_GLOB_REGEX = /^\//;
function formatGlob(glob) {
  return glob.trim().replace(EXCLUDED_GLOB_REGEX, "").replace(ESCAPED_NEGATIVE_GLOB_REGEX, "!").replace(DIR_GLOB_REGEX, "/**").replace(NOT_FIXED_START_GLOB_REGEX, "**/$1").replace(FIXED_START_GLOB_REGEX, "");
}
function processGlobs(globs) {
  const globLists = initGlobLists();
  for (const glob of globs) {
    const isExcluded = !!glob.match(EXCLAMATION_START);
    const formattedGlob = formatGlob(glob);
    if (!formattedGlob) {
      continue;
    }
    if (isExcluded) {
      globLists.excluded.push(formattedGlob);
    } else {
      if (formattedGlob.endsWith("/**")) {
        globLists.includedDirs.push(formattedGlob.replace(DIR_GLOB_REGEX, ""));
      }
      globLists.included.push(formattedGlob);
      globLists.originalIncluded.push(glob.trim());
    }
  }
  return globLists;
}
var COMMENT_OR_EMPTY_REGEX = /^\s*(#|$)/;
async function parseGlobsFile(filePath) {
  let fileContents;
  try {
    fileContents = (await fsAsync.readFile(filePath, { encoding: "utf-8" })).toString() || "";
  } catch (error) {
    console.error(`Failed to read glob file (${filePath})`);
    throw error;
  }
  const fileGlobs = fileContents.split(/\r?\n/).filter((line) => !line.match(COMMENT_OR_EMPTY_REGEX)) || [];
  return processGlobs(fileGlobs);
}
async function parseDefaultGlobsFile() {
  return parseGlobsFile(DEFAULT_GLOBS_FILE_PATH);
}
async function getGlobLists({
  noDefaults,
  globFile,
  globs
}) {
  const globListsToMerge = [];
  if (!noDefaults) {
    const defaultGlobLists = await parseDefaultGlobsFile();
    globListsToMerge.push(defaultGlobLists);
  }
  if (globFile && await fileExists(globFile)) {
    const userFileGlobLists = await parseGlobsFile(globFile);
    globListsToMerge.push(userFileGlobLists);
  }
  if (globs == null ? void 0 : globs.length) {
    const argGlobsLists = processGlobs(globs);
    globListsToMerge.push(argGlobsLists);
  }
  return globListsToMerge.reduce(
    (mergedGlobLists, globLists) => mergeGlobLists(mergedGlobLists, globLists),
    initGlobLists()
  );
}
async function findFilesByGlobLists(directory, globLists) {
  const { included, includedDirs, excluded } = toAbsoluteGlobLists(
    optimizeGlobLists(globLists),
    directory
  );
  const picoOptions = { ignore: excluded };
  const checkDir = (includedDirs == null ? void 0 : includedDirs.length) ? makeGlobMatcher(includedDirs, picoOptions) : () => false;
  const checkFile = makeGlobMatcher(included, picoOptions);
  let filesToRemove = await crawlDirWithChecks([], directory, checkDir, checkFile);
  if (excluded.length) {
    filesToRemove = filesToRemove.filter((file) => checkFile(file));
  }
  return filesToRemove;
}

// src/analyze.ts
async function analyze(options = {}) {
  const mergedOptions = { ...sharedDefaultOptions, ...options };
  const { globs, noDefaults, globFile, directory } = mergedOptions;
  const nodeModulesPath = directory || sharedDefaultOptions.directory;
  const globLists = await getGlobLists({ globs, noDefaults, globFile });
  const includedFiles = await findFilesByGlobLists(nodeModulesPath, globLists);
  const defaultGlobs = toAbsoluteGlobLists(
    optimizeGlobLists(await parseDefaultGlobsFile()),
    nodeModulesPath
  );
  const includedByDefaultMatcher = makeGlobMatcher(defaultGlobs.included, {
    ignore: defaultGlobs.excluded
  });
  const globMatchers = globLists.included.map((glob, index) => {
    const absoluteGlob = toPosixPath(nodeModulesPath) + "/" + glob;
    const matcher = makeGlobMatcher(absoluteGlob);
    return { original: globLists.originalIncluded[index], derived: absoluteGlob, matcher };
  });
  const analyzedResults = includedFiles.map((filePath) => {
    const includedByDefault = includedByDefaultMatcher(filePath);
    const includedByGlobs = [];
    for (const { original, derived, matcher } of globMatchers) {
      if (matcher(filePath)) {
        includedByGlobs.push({ original, derived });
      }
    }
    return { filePath, includedByDefault, includedByGlobs };
  });
  return analyzedResults;
}

// src/clean.ts
var defaultCleanOptions = {
  ...sharedDefaultOptions,
  dryRun: false,
  keepEmpty: false
};
async function clean(options = {}) {
  const mergedOptions = { ...defaultCleanOptions, ...options };
  const { globs, noDefaults, globFile, directory, dryRun, keepEmpty } = mergedOptions;
  const globLists = await getGlobLists({ globs, noDefaults, globFile });
  const files = await findFilesByGlobLists(directory, globLists);
  const reducedSize = await removeFiles(files, { dryRun });
  const removedEmptyDirs = dryRun || keepEmpty ? 0 : await removeEmptyDirs(files);
  return { files, reducedSize, removedEmptyDirs };
}

export {
  fileDir,
  sharedDefaultOptions,
  analyze,
  defaultCleanOptions,
  clean
};
