import {
  analyze,
  clean,
  fileDir,
  sharedDefaultOptions
} from "../chunk-GXCQXUOT.js";

// src/cli/cli.ts
import { Builtins, Cli } from "clipanion";
import { createRequire } from "module";
import path from "path";

// src/cli/helpers/base.command.ts
import { Command, Option } from "clipanion";
var BaseCommand = class extends Command {
  constructor() {
    super(...arguments);
    this.directory = Option.String("-D,--directory", sharedDefaultOptions.directory, {
      description: "Path to node_modules"
    });
    this.globFile = Option.String("-f,--glob-file", sharedDefaultOptions.globFile, {
      description: "Path to a custom glob file"
    });
    this.noDefaults = Option.Boolean("-n,--no-defaults", sharedDefaultOptions.noDefaults, {
      description: "Only includes/excludes globs specified by a custom glob file or CLI arguments"
    });
    this.globs = Option.Rest({ name: "globs" });
  }
};

// src/cli/commands/analyze.command.ts
var AnalyzeCommand = class extends BaseCommand {
  async execute() {
    const analyzeResults = await analyze({
      globs: this.globs,
      noDefaults: this.noDefaults,
      globFile: this.globFile
    });
    console.log(JSON.stringify(analyzeResults, null, 2));
  }
};
AnalyzeCommand.paths = [["analyze"]];
AnalyzeCommand.usage = {
  description: "Helps determining why a file is included by the clean command without removing any files. Extra globs can be passed as positional args."
};

// src/cli/commands/clean.command.ts
import { Command as Command2, Option as Option2 } from "clipanion";
import prettyBytes from "pretty-bytes";
import prettyMs from "pretty-ms";

// src/cli/utils/terminal.ts
import readline from "readline";
import supportsColor from "supports-color";
function makeLogger({ disabled }) {
  if (disabled) {
    const noop = () => void 0;
    return Object.keys(console).reduce((disabledConsole, key) => {
      disabledConsole[key] = noop;
      return disabledConsole;
    }, {});
  }
  return console;
}
async function yesOrNo(query) {
  const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout
  });
  return new Promise(
    (resolve) => rl.question(query, (answer) => {
      rl.close();
      resolve(/ye?s?/i.test(answer));
    })
  );
}
function colorizer(colorCode) {
  if (!supportsColor.stdout) {
    return (text) => String(text);
  }
  return (text) => `\x1B[${colorCode}m${text}\x1B[0m`;
}
var bold = colorizer("1");
var green = colorizer("32");
var yellow = colorizer("33");

// src/cli/commands/clean.command.ts
var CleanCommand = class extends BaseCommand {
  constructor() {
    super(...arguments);
    this.keepEmpty = Option2.Boolean("-k,--keep-empty", false, {
      description: "Skips removing empty folders after removing contents"
    });
    this.dryRun = Option2.Boolean("-d,--dry-run", false, {
      description: "Logs files that would be removed without removing any files"
    });
    this.silent = Option2.Boolean("-s,--silent", false, {
      description: "Does not log anything to console (unless --json is enabled)"
    });
    this.yes = Option2.Boolean("-y,--yes", false, {
      description: "Skips the confirmation prompt at the start of the script"
    });
    this.json = Option2.Boolean("-j,--json", false, {
      description: "Output results as JSON"
    });
  }
  async execute() {
    const logger = makeLogger({ disabled: this.json || this.silent });
    logger.log(bold("clean-modules"), this.dryRun ? yellow("(dry run)") : "");
    if (!this.yes && !this.dryRun) {
      const warning = `
Preparing to clean node_modules at: ${this.directory}
Are you sure you want to continue? (Y/N) `;
      const confirmed = await yesOrNo(yellow(warning));
      if (!confirmed) {
        process.exit(0);
      }
    }
    logger.log("\nCleaning up node_modules...");
    const cleanupStart = (/* @__PURE__ */ new Date()).getTime();
    const { files, reducedSize, removedEmptyDirs } = await clean({
      globs: this.globs,
      noDefaults: this.noDefaults,
      globFile: this.globFile,
      directory: this.directory,
      dryRun: this.dryRun
    });
    const cleanupDuration = (/* @__PURE__ */ new Date()).getTime() - cleanupStart;
    logger.log(green(`Done in ${prettyMs(cleanupDuration)}!`));
    if (this.json) {
      const output = {
        removedFiles: files.length,
        reducedSize,
        removedEmptyDirs,
        duration: cleanupDuration,
        dryRun: this.dryRun
      };
      console.log(JSON.stringify(output, null, 2));
    } else {
      logger.log(bold("\nResults:"));
      logger.log("- size reduced:", green(prettyBytes(reducedSize || 0)));
      logger.log("- files removed:", green(files.length));
      logger.log("- empty dirs removed:", green(removedEmptyDirs || 0));
    }
  }
};
CleanCommand.paths = [["clean"], Command2.Default];
CleanCommand.usage = {
  description: "Default command. Removes unnecessary files to reduce the size of your node_modules directory. Extra globs can be passed as positional args."
};

// src/cli/cli.ts
var [_node, _app, ...args] = process.argv;
var esmRequire = createRequire(import.meta.url);
var cliDir = fileDir(import.meta.url);
var { name, version } = esmRequire(path.resolve(cliDir, "..", "..", "package.json"));
var cli = new Cli({
  binaryLabel: name,
  binaryVersion: version,
  binaryName: name
});
cli.register(Builtins.HelpCommand);
cli.register(Builtins.VersionCommand);
cli.register(CleanCommand);
cli.register(AnalyzeCommand);
cli.runExit(args);
